/****************************************************************************
** Meta object code from reading C++ file 'QmlUtils.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/QmlUtils.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QmlUtils.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8QmlUtilsE_t {};
} // unnamed namespace

template <> constexpr inline auto QmlUtils::qt_create_metaobjectdata<qt_meta_tag_ZN8QmlUtilsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QmlUtils",
        "systemCountryCode",
        "",
        "getResourcePath",
        "resourceName",
        "applicationWebsiteUrl",
        "QUrl",
        "providerDetailsUrl",
        "providerJid",
        "invitationUrlString",
        "uri",
        "jid",
        "uriString",
        "groupChatUri",
        "groupChatJid",
        "groupChatUriString",
        "validateEncryptionKeyId",
        "keyId",
        "displayableEncryptionKeyId",
        "removeNewLinesFromString",
        "input",
        "quote",
        "text",
        "copyToClipboard",
        "url",
        "QImage",
        "image",
        "formattedDataSize",
        "fileSize",
        "userColor",
        "QColor",
        "id",
        "name",
        "osmUserAgent",
        "geoUri",
        "QGeoCoordinate",
        "geoCoordinate",
        "messageBubblePaddingCharacter",
        "emojiPrefix",
        "emojiSeparator",
        "groupChatUserMentionPrefix",
        "groupChatUserMentionSeparator",
        "versionString",
        "applicationDisplayName",
        "applicationSourceCodeUrl",
        "issueTrackingUrl",
        "donationUrl",
        "mastodonUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'systemCountryCode'
        QtMocHelpers::MethodData<QString()>(1, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'getResourcePath'
        QtMocHelpers::MethodData<QString(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 4 },
        }}),
        // Method 'applicationWebsiteUrl'
        QtMocHelpers::MethodData<QUrl()>(5, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Method 'providerDetailsUrl'
        QtMocHelpers::MethodData<QUrl(const QString &)>(7, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 8 },
        }}),
        // Method 'invitationUrlString'
        QtMocHelpers::MethodData<QString(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 10 },
        }}),
        // Method 'jid'
        QtMocHelpers::MethodData<QString(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 12 },
        }}),
        // Method 'groupChatUri'
        QtMocHelpers::MethodData<QUrl(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 14 },
        }}),
        // Method 'groupChatUriString'
        QtMocHelpers::MethodData<QString(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 14 },
        }}),
        // Method 'validateEncryptionKeyId'
        QtMocHelpers::MethodData<bool(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 },
        }}),
        // Method 'displayableEncryptionKeyId'
        QtMocHelpers::MethodData<QString(QString)>(18, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 17 },
        }}),
        // Method 'removeNewLinesFromString'
        QtMocHelpers::MethodData<QString(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 20 },
        }}),
        // Method 'quote'
        QtMocHelpers::MethodData<QString(QString)>(21, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'copyToClipboard'
        QtMocHelpers::MethodData<void(const QUrl &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 24 },
        }}),
        // Method 'copyToClipboard'
        QtMocHelpers::MethodData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'copyToClipboard'
        QtMocHelpers::MethodData<void(const QImage &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Method 'formattedDataSize'
        QtMocHelpers::MethodData<QString(qint64)>(27, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::LongLong, 28 },
        }}),
        // Method 'userColor'
        QtMocHelpers::MethodData<QColor(const QString &, const QString &)>(29, 2, QMC::AccessPublic, 0x80000000 | 30, {{
            { QMetaType::QString, 31 }, { QMetaType::QString, 32 },
        }}),
        // Method 'osmUserAgent'
        QtMocHelpers::MethodData<QString()>(33, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'geoUri'
        QtMocHelpers::MethodData<QString(const QGeoCoordinate &)>(34, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Method 'geoCoordinate'
        QtMocHelpers::MethodData<QGeoCoordinate(const QString &)>(36, 2, QMC::AccessPublic, 0x80000000 | 35, {{
            { QMetaType::QString, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'messageBubblePaddingCharacter'
        QtMocHelpers::PropertyData<QChar>(37, QMetaType::QChar, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'emojiPrefix'
        QtMocHelpers::PropertyData<QChar>(38, QMetaType::QChar, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'emojiSeparator'
        QtMocHelpers::PropertyData<QChar>(39, QMetaType::QChar, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'groupChatUserMentionPrefix'
        QtMocHelpers::PropertyData<QChar>(40, QMetaType::QChar, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'groupChatUserMentionSeparator'
        QtMocHelpers::PropertyData<QChar>(41, QMetaType::QChar, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'versionString'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'applicationDisplayName'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'applicationWebsiteUrl'
        QtMocHelpers::PropertyData<QUrl>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'applicationSourceCodeUrl'
        QtMocHelpers::PropertyData<QUrl>(44, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'issueTrackingUrl'
        QtMocHelpers::PropertyData<QUrl>(45, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'donationUrl'
        QtMocHelpers::PropertyData<QUrl>(46, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'mastodonUrl'
        QtMocHelpers::PropertyData<QUrl>(47, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QmlUtils, qt_meta_tag_ZN8QmlUtilsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QmlUtils::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QmlUtilsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QmlUtilsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8QmlUtilsE_t>.metaTypes,
    nullptr
} };

void QmlUtils::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QmlUtils *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->systemCountryCode();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->getResourcePath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QUrl _r = _t->applicationWebsiteUrl();
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 3: { QUrl _r = _t->providerDetailsUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->invitationUrlString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->jid((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 6: { QUrl _r = _t->groupChatUri((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->groupChatUriString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->validateEncryptionKeyId((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->displayableEncryptionKeyId((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->removeNewLinesFromString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->quote((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->copyToClipboard((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 13: _t->copyToClipboard((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->copyToClipboard((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 15: { QString _r = _t->formattedDataSize((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 16: { QColor _r = _t->userColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QColor*>(_a[0]) = std::move(_r); }  break;
        case 17: { QString _r = _t->osmUserAgent();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 18: { QString _r = _t->geoUri((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 19: { QGeoCoordinate _r = _t->geoCoordinate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QChar*>(_v) = _t->messageBubblePaddingCharacter(); break;
        case 1: *reinterpret_cast<QChar*>(_v) = _t->emojiPrefix(); break;
        case 2: *reinterpret_cast<QChar*>(_v) = _t->emojiSeparator(); break;
        case 3: *reinterpret_cast<QChar*>(_v) = _t->groupChatUserMentionPrefix(); break;
        case 4: *reinterpret_cast<QChar*>(_v) = _t->groupChatUserMentionSeparator(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->versionString(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->applicationDisplayName(); break;
        case 7: *reinterpret_cast<QUrl*>(_v) = _t->applicationWebsiteUrl(); break;
        case 8: *reinterpret_cast<QUrl*>(_v) = _t->applicationSourceCodeUrl(); break;
        case 9: *reinterpret_cast<QUrl*>(_v) = _t->issueTrackingUrl(); break;
        case 10: *reinterpret_cast<QUrl*>(_v) = _t->donationUrl(); break;
        case 11: *reinterpret_cast<QUrl*>(_v) = _t->mastodonUrl(); break;
        default: break;
        }
    }
}

const QMetaObject *QmlUtils::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QmlUtils::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QmlUtilsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QmlUtils::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
