/****************************************************************************
** Meta object code from reading C++ file 'MessageComposition.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/MessageComposition.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MessageComposition.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18MessageCompositionE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageComposition::qt_create_metaobjectdata<qt_meta_tag_ZN18MessageCompositionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageComposition",
        "accountJidChanged",
        "",
        "chatJidChanged",
        "replaceIdChanged",
        "replyToJidChanged",
        "replyToGroupChatParticipantIdChanged",
        "replyToNameChanged",
        "replyIdChanged",
        "originalReplyIdChanged",
        "replyQuoteChanged",
        "bodyChanged",
        "originalBodyChanged",
        "isSpoilerChanged",
        "spoilerHintChanged",
        "isDraftChanged",
        "isForwardingChanged",
        "draftSaved",
        "preparedForNewChat",
        "send",
        "customBody",
        "sendGeoUri",
        "QGeoCoordinate",
        "geoCoordinate",
        "correct",
        "clear",
        "chatController",
        "ChatController*",
        "fileSelectionModel",
        "FileSelectionModel*",
        "replaceId",
        "replyToJid",
        "replyToGroupChatParticipantId",
        "replyToName",
        "replyId",
        "originalReplyId",
        "replyQuote",
        "body",
        "originalBody",
        "isSpoiler",
        "spoilerHint",
        "isDraft",
        "isForwarding"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountJidChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'chatJidChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replaceIdChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replyToJidChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replyToGroupChatParticipantIdChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replyToNameChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replyIdChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'originalReplyIdChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replyQuoteChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bodyChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'originalBodyChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isSpoilerChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spoilerHintChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isDraftChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isForwardingChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'draftSaved'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preparedForNewChat'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'send'
        QtMocHelpers::MethodData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Method 'send'
        QtMocHelpers::MethodData<void()>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'sendGeoUri'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Method 'correct'
        QtMocHelpers::MethodData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'chatController'
        QtMocHelpers::PropertyData<ChatController*>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'fileSelectionModel'
        QtMocHelpers::PropertyData<FileSelectionModel*>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'replaceId'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'replyToJid'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'replyToGroupChatParticipantId'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'replyToName'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'replyId'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'originalReplyId'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'replyQuote'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'body'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'originalBody'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'isSpoiler'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 11),
        // property 'spoilerHint'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'isDraft'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'isForwarding'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags, 14),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageComposition, qt_meta_tag_ZN18MessageCompositionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageComposition::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18MessageCompositionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18MessageCompositionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18MessageCompositionE_t>.metaTypes,
    nullptr
} };

void MessageComposition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MessageComposition *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountJidChanged(); break;
        case 1: _t->chatJidChanged(); break;
        case 2: _t->replaceIdChanged(); break;
        case 3: _t->replyToJidChanged(); break;
        case 4: _t->replyToGroupChatParticipantIdChanged(); break;
        case 5: _t->replyToNameChanged(); break;
        case 6: _t->replyIdChanged(); break;
        case 7: _t->originalReplyIdChanged(); break;
        case 8: _t->replyQuoteChanged(); break;
        case 9: _t->bodyChanged(); break;
        case 10: _t->originalBodyChanged(); break;
        case 11: _t->isSpoilerChanged(); break;
        case 12: _t->spoilerHintChanged(); break;
        case 13: _t->isDraftChanged(); break;
        case 14: _t->isForwardingChanged(); break;
        case 15: _t->draftSaved(); break;
        case 16: _t->preparedForNewChat(); break;
        case 17: _t->send((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->send(); break;
        case 19: _t->sendGeoUri((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 20: _t->correct(); break;
        case 21: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::accountJidChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::chatJidChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::replaceIdChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::replyToJidChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::replyToGroupChatParticipantIdChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::replyToNameChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::replyIdChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::originalReplyIdChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::replyQuoteChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::bodyChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::originalBodyChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::isSpoilerChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::spoilerHintChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::isDraftChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::isForwardingChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::draftSaved, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageComposition::*)()>(_a, &MessageComposition::preparedForNewChat, 16))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileSelectionModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<ChatController**>(_v) = _t->m_chatController; break;
        case 1: *reinterpret_cast<FileSelectionModel**>(_v) = _t->m_fileSelectionModel; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->replaceId(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->replyToJid(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->replyToGroupChatParticipantId(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->replyToName(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->replyId(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->originalReplyId(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->replyQuote(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->body(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->originalBody(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isSpoiler(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->spoilerHint(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isDraft(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isForwarding(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setChatController(*reinterpret_cast<ChatController**>(_v)); break;
        case 2: _t->setReplaceId(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setReplyToJid(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setReplyToGroupChatParticipantId(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setReplyToName(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setReplyId(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setOriginalReplyId(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setReplyQuote(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setBody(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setOriginalBody(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setSpoiler(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setSpoilerHint(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setIsDraft(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MessageComposition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageComposition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18MessageCompositionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MessageComposition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void MessageComposition::accountJidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MessageComposition::chatJidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MessageComposition::replaceIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MessageComposition::replyToJidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MessageComposition::replyToGroupChatParticipantIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MessageComposition::replyToNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MessageComposition::replyIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MessageComposition::originalReplyIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MessageComposition::replyQuoteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void MessageComposition::bodyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void MessageComposition::originalBodyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void MessageComposition::isSpoilerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void MessageComposition::spoilerHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void MessageComposition::isDraftChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void MessageComposition::isForwardingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void MessageComposition::draftSaved()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void MessageComposition::preparedForNewChat()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}
namespace {
struct qt_meta_tag_ZN18FileSelectionModelE_t {};
} // unnamed namespace

template <> constexpr inline auto FileSelectionModel::qt_create_metaobjectdata<qt_meta_tag_ZN18FileSelectionModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileSelectionModel",
        "selectFile",
        "",
        "addVoiceMessageFile",
        "QUrl",
        "localFileUrl",
        "addFile",
        "isNew",
        "removeFile",
        "index",
        "deleteNewFiles",
        "clear"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'selectFile'
        QtMocHelpers::MethodData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'addVoiceMessageFile'
        QtMocHelpers::MethodData<void(const QUrl &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Method 'addFile'
        QtMocHelpers::MethodData<void(const QUrl &, bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Bool, 7 },
        }}),
        // Method 'addFile'
        QtMocHelpers::MethodData<void(const QUrl &)>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Method 'removeFile'
        QtMocHelpers::MethodData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Method 'deleteNewFiles'
        QtMocHelpers::MethodData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileSelectionModel, qt_meta_tag_ZN18FileSelectionModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileSelectionModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18FileSelectionModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18FileSelectionModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18FileSelectionModelE_t>.metaTypes,
    nullptr
} };

void FileSelectionModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileSelectionModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectFile(); break;
        case 1: _t->addVoiceMessageFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 2: _t->addFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->addFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->removeFile((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->deleteNewFiles(); break;
        case 6: _t->clear(); break;
        default: ;
        }
    }
}

const QMetaObject *FileSelectionModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileSelectionModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18FileSelectionModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int FileSelectionModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
