/****************************************************************************
** Meta object code from reading C++ file 'ProviderModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/ProviderModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ProviderModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ProviderModelE_t {};
} // unnamed namespace

template <> constexpr inline auto ProviderModel::qt_create_metaobjectdata<qt_meta_tag_ZN13ProviderModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ProviderModel",
        "data",
        "QVariant",
        "",
        "row",
        "ProviderModel::Role",
        "role",
        "providerFromBareJid",
        "Provider",
        "jid",
        "randomlyChooseIndex",
        "QList<int>",
        "excludedIndexes",
        "providersMatchingSystemLocaleOnly",
        "providersMatchingSystemLocaleMinimumCount",
        "maximumBusFactor",
        "availableFlags",
        "availableOrganizations",
        "Role",
        "Display",
        "IsCustomProvider",
        "Jid",
        "SupportsInBandRegistration",
        "RegistrationWebPage",
        "Languages",
        "Flags",
        "FlagsText",
        "Website",
        "BusFactor",
        "Organization",
        "SupportsPasswordReset",
        "HostedGreen",
        "Since",
        "SinceText",
        "HttpUploadFileSize",
        "HttpUploadFileSizeText",
        "HttpUploadTotalSize",
        "HttpUploadStorageDuration",
        "MessageStorageDuration",
        "MessageStorageDurationText",
        "FreeOfCharge"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'data'
        QtMocHelpers::MethodData<QVariant(int, ProviderModel::Role) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::Int, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Method 'providerFromBareJid'
        QtMocHelpers::MethodData<Provider(const QString &) const>(7, 3, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 9 },
        }}),
        // Method 'randomlyChooseIndex'
        QtMocHelpers::MethodData<int(const QList<int> &, bool) const>(10, 3, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Method 'randomlyChooseIndex'
        QtMocHelpers::MethodData<int(const QList<int> &) const>(10, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Method 'randomlyChooseIndex'
        QtMocHelpers::MethodData<int() const>(10, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'providersMatchingSystemLocaleMinimumCount'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'maximumBusFactor'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'availableFlags'
        QtMocHelpers::PropertyData<QList<QString>>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'availableOrganizations'
        QtMocHelpers::PropertyData<QList<QString>>(17, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Role'
        QtMocHelpers::EnumData<enum Role>(18, 18, QMC::EnumIsScoped).add({
            {   19, Role::Display },
            {   20, Role::IsCustomProvider },
            {   21, Role::Jid },
            {   22, Role::SupportsInBandRegistration },
            {   23, Role::RegistrationWebPage },
            {   24, Role::Languages },
            {   25, Role::Flags },
            {   26, Role::FlagsText },
            {   27, Role::Website },
            {   28, Role::BusFactor },
            {   29, Role::Organization },
            {   30, Role::SupportsPasswordReset },
            {   31, Role::HostedGreen },
            {   32, Role::Since },
            {   33, Role::SinceText },
            {   34, Role::HttpUploadFileSize },
            {   35, Role::HttpUploadFileSizeText },
            {   36, Role::HttpUploadTotalSize },
            {   37, Role::HttpUploadStorageDuration },
            {   38, Role::MessageStorageDuration },
            {   39, Role::MessageStorageDurationText },
            {   40, Role::FreeOfCharge },
        }),
    };
    return QtMocHelpers::metaObjectData<ProviderModel, qt_meta_tag_ZN13ProviderModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ProviderModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProviderModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProviderModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ProviderModelE_t>.metaTypes,
    nullptr
} };

void ProviderModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProviderModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QVariant _r = _t->data((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ProviderModel::Role>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 1: { Provider _r = _t->providerFromBareJid((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Provider*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->randomlyChooseIndex((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 3: { int _r = _t->randomlyChooseIndex((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->randomlyChooseIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->providersMatchingSystemLocaleMinimumCount(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->m_maximumBusFactor; break;
        case 2: *reinterpret_cast<QList<QString>*>(_v) = _t->m_availableFlags; break;
        case 3: *reinterpret_cast<QList<QString>*>(_v) = _t->m_availableOrganizations; break;
        default: break;
        }
    }
}

const QMetaObject *ProviderModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProviderModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProviderModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int ProviderModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
