/****************************************************************************
** Meta object code from reading C++ file 'PublicGroupChat.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/PublicGroupChat.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PublicGroupChat.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PublicGroupChatE_t {};
} // unnamed namespace

template <> constexpr inline auto PublicGroupChat::qt_create_metaobjectdata<qt_meta_tag_ZN15PublicGroupChatE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PublicGroupChat",
        "address",
        "users",
        "isOpen",
        "name",
        "description",
        "languages",
        "json",
        "QJsonObject"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'address'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'users'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isOpen'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'languages'
        QtMocHelpers::PropertyData<QStringList>(6, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'json'
        QtMocHelpers::PropertyData<QJsonObject>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PublicGroupChat, qt_meta_tag_ZN15PublicGroupChatE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PublicGroupChat::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PublicGroupChatE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PublicGroupChatE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PublicGroupChatE_t>.metaTypes,
    nullptr
} };

void PublicGroupChat::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<PublicGroupChat *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->address(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->users(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isOpen(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->languages(); break;
        case 6: *reinterpret_cast<QJsonObject*>(_v) = _t->toJson(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAddress(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setUsers(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setIsOpen(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setDescription(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setLanguages(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
