/****************************************************************************
** Meta object code from reading C++ file 'InputValidator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/InputValidator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'InputValidator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14InputValidatorE_t {};
} // unnamed namespace

template <> constexpr inline auto InputValidator::qt_create_metaobjectdata<qt_meta_tag_ZN14InputValidatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "InputValidator",
        "patternsChanged",
        "",
        "InputValidator::Patterns",
        "patterns",
        "stateChanged",
        "QValidator::State",
        "state",
        "Pattern",
        "None",
        "Empty",
        "NotEmpty",
        "Username",
        "Hostname",
        "Jid",
        "Port",
        "Patterns"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'patternsChanged'
        QtMocHelpers::SignalData<void(InputValidator::Patterns)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(QValidator::State)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'patterns'
        QtMocHelpers::PropertyData<InputValidator::Patterns>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'state'
        QtMocHelpers::PropertyData<QValidator::State>(7, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Pattern'
        QtMocHelpers::EnumData<enum Pattern>(8, 8, QMC::EnumIsScoped).add({
            {    9, Pattern::None },
            {   10, Pattern::Empty },
            {   11, Pattern::NotEmpty },
            {   12, Pattern::Username },
            {   13, Pattern::Hostname },
            {   14, Pattern::Jid },
            {   15, Pattern::Port },
        }),
        // flag 'Patterns'
        QtMocHelpers::EnumData<Patterns>(16, 8, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {    9, Pattern::None },
            {   10, Pattern::Empty },
            {   11, Pattern::NotEmpty },
            {   12, Pattern::Username },
            {   13, Pattern::Hostname },
            {   14, Pattern::Jid },
            {   15, Pattern::Port },
        }),
    };
    return QtMocHelpers::metaObjectData<InputValidator, qt_meta_tag_ZN14InputValidatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN14InputValidatorE[] = {
    QMetaObject::SuperData::link<QValidator::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject InputValidator::staticMetaObject = { {
    QMetaObject::SuperData::link<QRegularExpressionValidator::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14InputValidatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14InputValidatorE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN14InputValidatorE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14InputValidatorE_t>.metaTypes,
    nullptr
} };

void InputValidator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<InputValidator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->patternsChanged((*reinterpret_cast<std::add_pointer_t<InputValidator::Patterns>>(_a[1]))); break;
        case 1: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QValidator::State>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (InputValidator::*)(InputValidator::Patterns )>(_a, &InputValidator::patternsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputValidator::*)(QValidator::State )>(_a, &InputValidator::stateChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<InputValidator::Patterns*>(_v) = _t->patterns(); break;
        case 1: *reinterpret_cast<QValidator::State*>(_v) = _t->state(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPatterns(*reinterpret_cast<InputValidator::Patterns*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *InputValidator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InputValidator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14InputValidatorE_t>.strings))
        return static_cast<void*>(this);
    return QRegularExpressionValidator::qt_metacast(_clname);
}

int InputValidator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRegularExpressionValidator::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void InputValidator::patternsChanged(InputValidator::Patterns _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void InputValidator::stateChanged(QValidator::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
