/****************************************************************************
** Meta object code from reading C++ file 'PresenceCache.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/PresenceCache.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PresenceCache.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8PresenceE_t {};
} // unnamed namespace

template <> constexpr inline auto Presence::qt_create_metaobjectdata<qt_meta_tag_ZN8PresenceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Presence",
        "Availability",
        "Offline",
        "Online",
        "Away",
        "XA",
        "DND",
        "Chat"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Availability'
        QtMocHelpers::EnumData<enum Availability>(1, 1, QMC::EnumFlags{}).add({
            {    2, Availability::Offline },
            {    3, Availability::Online },
            {    4, Availability::Away },
            {    5, Availability::XA },
            {    6, Availability::DND },
            {    7, Availability::Chat },
        }),
    };
    return QtMocHelpers::metaObjectData<Presence, qt_meta_tag_ZN8PresenceE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Presence::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PresenceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PresenceE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8PresenceE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN13PresenceCacheE_t {};
} // unnamed namespace

template <> constexpr inline auto PresenceCache::qt_create_metaobjectdata<qt_meta_tag_ZN13PresenceCacheE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PresenceCache",
        "presenceChanged",
        "",
        "PresenceCache::ChangeType",
        "type",
        "jid",
        "resource",
        "presencesCleared",
        "ChangeType",
        "Connected",
        "Disconnected",
        "Updated"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'presenceChanged'
        QtMocHelpers::SignalData<void(PresenceCache::ChangeType, const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'presencesCleared'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ChangeType'
        QtMocHelpers::EnumData<enum ChangeType>(8, 8, QMC::EnumFlags{}).add({
            {    9, ChangeType::Connected },
            {   10, ChangeType::Disconnected },
            {   11, ChangeType::Updated },
        }),
    };
    return QtMocHelpers::metaObjectData<PresenceCache, qt_meta_tag_ZN13PresenceCacheE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PresenceCache::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PresenceCacheE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PresenceCacheE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13PresenceCacheE_t>.metaTypes,
    nullptr
} };

void PresenceCache::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PresenceCache *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->presenceChanged((*reinterpret_cast<std::add_pointer_t<PresenceCache::ChangeType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->presencesCleared(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PresenceCache::*)(PresenceCache::ChangeType , const QString & , const QString & )>(_a, &PresenceCache::presenceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PresenceCache::*)()>(_a, &PresenceCache::presencesCleared, 1))
            return;
    }
}

const QMetaObject *PresenceCache::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PresenceCache::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PresenceCacheE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PresenceCache::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void PresenceCache::presenceChanged(PresenceCache::ChangeType _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void PresenceCache::presencesCleared()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN20UserResourcesWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto UserResourcesWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN20UserResourcesWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UserResourcesWatcher",
        "jidChanged",
        "",
        "resourcesCountChanged",
        "presenceCache",
        "PresenceCache*",
        "jid",
        "resourcesCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'jidChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resourcesCountChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'presenceCache'
        QtMocHelpers::PropertyData<PresenceCache*>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'jid'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'resourcesCount'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UserResourcesWatcher, qt_meta_tag_ZN20UserResourcesWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UserResourcesWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20UserResourcesWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20UserResourcesWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20UserResourcesWatcherE_t>.metaTypes,
    nullptr
} };

void UserResourcesWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UserResourcesWatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->jidChanged(); break;
        case 1: _t->resourcesCountChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UserResourcesWatcher::*)()>(_a, &UserResourcesWatcher::jidChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UserResourcesWatcher::*)()>(_a, &UserResourcesWatcher::resourcesCountChanged, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PresenceCache* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<PresenceCache**>(_v) = _t->m_presenceCache; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->jid(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->resourcesCount(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPresenceCache(*reinterpret_cast<PresenceCache**>(_v)); break;
        case 1: _t->setJid(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *UserResourcesWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UserResourcesWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20UserResourcesWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int UserResourcesWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void UserResourcesWatcher::jidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void UserResourcesWatcher::resourcesCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
