/****************************************************************************
** Meta object code from reading C++ file 'RosterController.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/RosterController.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RosterController.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16RosterControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto RosterController::qt_create_metaobjectdata<qt_meta_tag_ZN16RosterControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RosterController",
        "contactAdditionFailed",
        "",
        "jid",
        "presenceSubscriptionRequestReceived",
        "QXmppPresence",
        "request",
        "groupsChanged",
        "addContactWithUri",
        "ContactAdditionWithUriResult",
        "uriString",
        "name",
        "addContact",
        "automaticInitialAddition",
        "renameContact",
        "newContactName",
        "subscribeToPresence",
        "contactJid",
        "refuseSubscriptionToPresence",
        "updateGroup",
        "oldGroup",
        "newGroup",
        "removeGroup",
        "group",
        "updateGroups",
        "groups",
        "setChatStateSendingEnabled",
        "chatStateSendingEnabled",
        "setReadMarkerSendingEnabled",
        "readMarkerSendingEnabled",
        "setNotificationRule",
        "RosterItem::NotificationRule",
        "notificationRule",
        "setAutomaticMediaDownloadsRule",
        "RosterItem::AutomaticMediaDownloadsRule",
        "rule",
        "removeContact",
        "AddingContact",
        "ContactExists",
        "InvalidUri"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'contactAdditionFailed'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'presenceSubscriptionRequestReceived'
        QtMocHelpers::SignalData<void(const QXmppPresence &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'groupsChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'addContactWithUri'
        QtMocHelpers::MethodData<enum ContactAdditionWithUriResult(const QString &, const QString &)>(8, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Method 'addContactWithUri'
        QtMocHelpers::MethodData<enum ContactAdditionWithUriResult(const QString &)>(8, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 9, {{
            { QMetaType::QString, 10 },
        }}),
        // Method 'addContact'
        QtMocHelpers::MethodData<void(const QString &, const QString &, bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 11 }, { QMetaType::Bool, 13 },
        }}),
        // Method 'addContact'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 11 },
        }}),
        // Method 'addContact'
        QtMocHelpers::MethodData<void(const QString &)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Method 'renameContact'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 15 },
        }}),
        // Method 'subscribeToPresence'
        QtMocHelpers::MethodData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Method 'refuseSubscriptionToPresence'
        QtMocHelpers::MethodData<bool(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 },
        }}),
        // Method 'updateGroup'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::QString, 21 },
        }}),
        // Method 'removeGroup'
        QtMocHelpers::MethodData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Method 'updateGroups'
        QtMocHelpers::MethodData<void(const QString &, const QList<QString> &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QStringList, 25 },
        }}),
        // Method 'setChatStateSendingEnabled'
        QtMocHelpers::MethodData<void(const QString &, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 27 },
        }}),
        // Method 'setReadMarkerSendingEnabled'
        QtMocHelpers::MethodData<void(const QString &, bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 29 },
        }}),
        // Method 'setNotificationRule'
        QtMocHelpers::MethodData<void(const QString &, RosterItem::NotificationRule)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 31, 32 },
        }}),
        // Method 'setAutomaticMediaDownloadsRule'
        QtMocHelpers::MethodData<void(const QString &, RosterItem::AutomaticMediaDownloadsRule)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 34, 35 },
        }}),
        // Method 'removeContact'
        QtMocHelpers::MethodData<void(const QString &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'groups'
        QtMocHelpers::PropertyData<QList<QString>>(25, QMetaType::QStringList, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ContactAdditionWithUriResult'
        QtMocHelpers::EnumData<enum ContactAdditionWithUriResult>(9, 9, QMC::EnumIsScoped).add({
            {   37, ContactAdditionWithUriResult::AddingContact },
            {   38, ContactAdditionWithUriResult::ContactExists },
            {   39, ContactAdditionWithUriResult::InvalidUri },
        }),
    };
    return QtMocHelpers::metaObjectData<RosterController, qt_meta_tag_ZN16RosterControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RosterController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16RosterControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16RosterControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16RosterControllerE_t>.metaTypes,
    nullptr
} };

void RosterController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RosterController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contactAdditionFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->presenceSubscriptionRequestReceived((*reinterpret_cast<std::add_pointer_t<QXmppPresence>>(_a[1]))); break;
        case 2: _t->groupsChanged(); break;
        case 3: { enum ContactAdditionWithUriResult _r = _t->addContactWithUri((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<enum ContactAdditionWithUriResult*>(_a[0]) = std::move(_r); }  break;
        case 4: { enum ContactAdditionWithUriResult _r = _t->addContactWithUri((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<enum ContactAdditionWithUriResult*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->addContact((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 6: _t->addContact((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->addContact((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->renameContact((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->subscribeToPresence((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: { bool _r = _t->refuseSubscriptionToPresence((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->updateGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->removeGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->updateGroups((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QString>>>(_a[2]))); break;
        case 14: _t->setChatStateSendingEnabled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 15: _t->setReadMarkerSendingEnabled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 16: _t->setNotificationRule((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<RosterItem::NotificationRule>>(_a[2]))); break;
        case 17: _t->setAutomaticMediaDownloadsRule((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<RosterItem::AutomaticMediaDownloadsRule>>(_a[2]))); break;
        case 18: _t->removeContact((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< RosterItem::NotificationRule >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< RosterItem::AutomaticMediaDownloadsRule >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RosterController::*)(const QString & )>(_a, &RosterController::contactAdditionFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterController::*)(const QXmppPresence & )>(_a, &RosterController::presenceSubscriptionRequestReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterController::*)()>(_a, &RosterController::groupsChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QString>*>(_v) = _t->groups(); break;
        default: break;
        }
    }
}

const QMetaObject *RosterController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RosterController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16RosterControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int RosterController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void RosterController::contactAdditionFailed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void RosterController::presenceSubscriptionRequestReceived(const QXmppPresence & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void RosterController::groupsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
