/****************************************************************************
** Meta object code from reading C++ file 'EmojiModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/EmojiModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EmojiModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5EmojiE_t {};
} // unnamed namespace

template <> constexpr inline auto Emoji::qt_create_metaobjectdata<qt_meta_tag_ZN5EmojiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Emoji",
        "unicode",
        "shortName",
        "group",
        "Emoji::Group",
        "Group",
        "Invalid",
        "Favorites",
        "People",
        "Nature",
        "Food",
        "Activity",
        "Travel",
        "Objects",
        "Symbols",
        "Flags"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'unicode'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'shortName'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'group'
        QtMocHelpers::PropertyData<Emoji::Group>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Group'
        QtMocHelpers::EnumData<enum Group>(5, 5, QMC::EnumIsScoped).add({
            {    6, Group::Invalid },
            {    7, Group::Favorites },
            {    8, Group::People },
            {    9, Group::Nature },
            {   10, Group::Food },
            {   11, Group::Activity },
            {   12, Group::Travel },
            {   13, Group::Objects },
            {   14, Group::Symbols },
            {   15, Group::Flags },
        }),
    };
    return QtMocHelpers::metaObjectData<Emoji, qt_meta_tag_ZN5EmojiE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Emoji::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5EmojiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5EmojiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5EmojiE_t>.metaTypes,
    nullptr
} };

void Emoji::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Emoji *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->unicode(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->shortName(); break;
        case 2: *reinterpret_cast<Emoji::Group*>(_v) = _t->group(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN10EmojiModelE_t {};
} // unnamed namespace

template <> constexpr inline auto EmojiModel::qt_create_metaobjectdata<qt_meta_tag_ZN10EmojiModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EmojiModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EmojiModel, qt_meta_tag_ZN10EmojiModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EmojiModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10EmojiModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10EmojiModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10EmojiModelE_t>.metaTypes,
    nullptr
} };

void EmojiModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EmojiModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *EmojiModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EmojiModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10EmojiModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int EmojiModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15EmojiProxyModelE_t {};
} // unnamed namespace

template <> constexpr inline auto EmojiProxyModel::qt_create_metaobjectdata<qt_meta_tag_ZN15EmojiProxyModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EmojiProxyModel",
        "groupChanged",
        "",
        "hasFavoriteEmojisChanged",
        "addFavoriteEmoji",
        "proxyRow",
        "group",
        "Emoji::Group",
        "hasFavoriteEmojis"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'groupChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasFavoriteEmojisChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'addFavoriteEmoji'
        QtMocHelpers::MethodData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'group'
        QtMocHelpers::PropertyData<Emoji::Group>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'hasFavoriteEmojis'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EmojiProxyModel, qt_meta_tag_ZN15EmojiProxyModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN15EmojiProxyModelE[] = {
    QMetaObject::SuperData::link<Emoji::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject EmojiProxyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15EmojiProxyModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15EmojiProxyModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN15EmojiProxyModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15EmojiProxyModelE_t>.metaTypes,
    nullptr
} };

void EmojiProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EmojiProxyModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->groupChanged(); break;
        case 1: _t->hasFavoriteEmojisChanged(); break;
        case 2: _t->addFavoriteEmoji((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EmojiProxyModel::*)()>(_a, &EmojiProxyModel::groupChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EmojiProxyModel::*)()>(_a, &EmojiProxyModel::hasFavoriteEmojisChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Emoji::Group*>(_v) = _t->group(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->hasFavoriteEmojis(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setGroup(*reinterpret_cast<Emoji::Group*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *EmojiProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EmojiProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15EmojiProxyModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int EmojiProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void EmojiProxyModel::groupChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void EmojiProxyModel::hasFavoriteEmojisChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
