/****************************************************************************
** Meta object code from reading C++ file 'AccountController.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/AccountController.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AccountController.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17AccountControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto AccountController::qt_create_metaobjectdata<qt_meta_tag_ZN17AccountControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AccountController",
        "accountsChanged",
        "",
        "accountAvailable",
        "noAccountAvailable",
        "migratingChanged",
        "allAccountsLoggedOutToQuit",
        "accountAdded",
        "Account*",
        "account",
        "accountRemoved",
        "jid",
        "createUninitializedAccount",
        "discardUninitializedAccount",
        "enabledAccounts",
        "QList<Account*>",
        "setActiveAccount",
        "resetActiveAccount",
        "startMigration",
        "logOutAllAccountsToQuit",
        "accounts",
        "migrating"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountAvailable'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noAccountAvailable'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'migratingChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'allAccountsLoggedOutToQuit'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'accountAdded'
        QtMocHelpers::SignalData<void(Account *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'accountRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'createUninitializedAccount'
        QtMocHelpers::MethodData<Account *()>(12, 2, QMC::AccessPublic, 0x80000000 | 8),
        // Method 'discardUninitializedAccount'
        QtMocHelpers::MethodData<void(Account *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Method 'account'
        QtMocHelpers::MethodData<Account *(const QString &) const>(9, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'enabledAccounts'
        QtMocHelpers::MethodData<QList<Account*>() const>(14, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Method 'setActiveAccount'
        QtMocHelpers::MethodData<void(Account *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Method 'resetActiveAccount'
        QtMocHelpers::MethodData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'startMigration'
        QtMocHelpers::MethodData<void(Account *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Method 'logOutAllAccountsToQuit'
        QtMocHelpers::MethodData<bool()>(19, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accounts'
        QtMocHelpers::PropertyData<QList<Account*>>(20, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'migrating'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AccountController, qt_meta_tag_ZN17AccountControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AccountController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AccountControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AccountControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17AccountControllerE_t>.metaTypes,
    nullptr
} };

void AccountController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountsChanged(); break;
        case 1: _t->accountAvailable(); break;
        case 2: _t->noAccountAvailable(); break;
        case 3: _t->migratingChanged(); break;
        case 4: _t->allAccountsLoggedOutToQuit(); break;
        case 5: _t->accountAdded((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 6: _t->accountRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: { Account* _r = _t->createUninitializedAccount();
            if (_a[0]) *reinterpret_cast<Account**>(_a[0]) = std::move(_r); }  break;
        case 8: _t->discardUninitializedAccount((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 9: { Account* _r = _t->account((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Account**>(_a[0]) = std::move(_r); }  break;
        case 10: { QList<Account*> _r = _t->enabledAccounts();
            if (_a[0]) *reinterpret_cast<QList<Account*>*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setActiveAccount((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 12: _t->resetActiveAccount(); break;
        case 13: _t->startMigration((*reinterpret_cast<std::add_pointer_t<Account*>>(_a[1]))); break;
        case 14: { bool _r = _t->logOutAllAccountsToQuit();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Account* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Account* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Account* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Account* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountController::*)()>(_a, &AccountController::accountsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountController::*)()>(_a, &AccountController::accountAvailable, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountController::*)()>(_a, &AccountController::noAccountAvailable, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountController::*)()>(_a, &AccountController::migratingChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountController::*)()>(_a, &AccountController::allAccountsLoggedOutToQuit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountController::*)(Account * )>(_a, &AccountController::accountAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountController::*)(const QString & )>(_a, &AccountController::accountRemoved, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<Account*> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<Account*>*>(_v) = _t->accounts(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->migrating(); break;
        default: break;
        }
    }
}

const QMetaObject *AccountController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AccountController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AccountControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AccountController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void AccountController::accountsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AccountController::accountAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void AccountController::noAccountAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void AccountController::migratingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AccountController::allAccountsLoggedOutToQuit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void AccountController::accountAdded(Account * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void AccountController::accountRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
