/****************************************************************************
** Meta object code from reading C++ file 'MediaUtils.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/MediaUtils.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MediaUtils.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MediaUtilsE_t {};
} // unnamed namespace

template <> constexpr inline auto MediaUtils::qt_create_metaobjectdata<qt_meta_tag_ZN10MediaUtilsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MediaUtils",
        "prettyDuration",
        "",
        "msecs",
        "durationMsecs",
        "isHttp",
        "content",
        "isGeoLocation",
        "locationCoordinate",
        "QGeoCoordinate",
        "localFileAvailable",
        "filePath",
        "urlFromClipboard",
        "QUrl",
        "deleteFile",
        "url",
        "deleteDownloadedFile",
        "localFilePath",
        "localFileUrl",
        "imageValid",
        "QImage",
        "image",
        "mimeType",
        "QMimeType",
        "data",
        "iconName",
        "newAudioFileUrl",
        "newImageFileUrl",
        "newVideoFileUrl",
        "openFile",
        "openFileInFolder",
        "newMediaLabel",
        "Enums::MessageType",
        "hint",
        "newMediaIconName",
        "label",
        "filterName",
        "filter",
        "namedFilter",
        "mimeTypes",
        "QList<QMimeType>",
        "messageType",
        "mimeTypeName",
        "hasAlphaChannel",
        "QVariant",
        "source"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'prettyDuration'
        QtMocHelpers::MethodData<QString(int)>(1, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 3 },
        }}),
        // Method 'prettyDuration'
        QtMocHelpers::MethodData<QString(int, int)>(1, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Method 'isHttp'
        QtMocHelpers::MethodData<bool(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'isGeoLocation'
        QtMocHelpers::MethodData<bool(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'locationCoordinate'
        QtMocHelpers::MethodData<QGeoCoordinate(const QString &)>(8, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'localFileAvailable'
        QtMocHelpers::MethodData<bool(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'urlFromClipboard'
        QtMocHelpers::MethodData<QUrl()>(12, 2, QMC::AccessPublic, 0x80000000 | 13),
        // Method 'deleteFile'
        QtMocHelpers::MethodData<void(const QUrl &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 15 },
        }}),
        // Method 'deleteDownloadedFile'
        QtMocHelpers::MethodData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Method 'localFilePath'
        QtMocHelpers::MethodData<QString(const QUrl &)>(17, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 13, 18 },
        }}),
        // Method 'localFileUrl'
        QtMocHelpers::MethodData<QUrl(const QString &)>(18, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::QString, 17 },
        }}),
        // Method 'imageValid'
        QtMocHelpers::MethodData<bool(const QImage &)>(19, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'mimeType'
        QtMocHelpers::MethodData<QMimeType(const QByteArray &)>(22, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { QMetaType::QByteArray, 24 },
        }}),
        // Method 'mimeType'
        QtMocHelpers::MethodData<QMimeType(const QString &)>(22, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'mimeType'
        QtMocHelpers::MethodData<QMimeType(const QUrl &)>(22, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { 0x80000000 | 13, 15 },
        }}),
        // Method 'iconName'
        QtMocHelpers::MethodData<QString(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'iconName'
        QtMocHelpers::MethodData<QString(const QUrl &)>(25, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 13, 15 },
        }}),
        // Method 'newAudioFileUrl'
        QtMocHelpers::MethodData<QUrl()>(26, 2, QMC::AccessPublic, 0x80000000 | 13),
        // Method 'newImageFileUrl'
        QtMocHelpers::MethodData<QUrl()>(27, 2, QMC::AccessPublic, 0x80000000 | 13),
        // Method 'newVideoFileUrl'
        QtMocHelpers::MethodData<QUrl()>(28, 2, QMC::AccessPublic, 0x80000000 | 13),
        // Method 'openFile'
        QtMocHelpers::MethodData<QUrl()>(29, 2, QMC::AccessPublic, 0x80000000 | 13),
        // Method 'openFileInFolder'
        QtMocHelpers::MethodData<void(const QUrl &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 18 },
        }}),
        // Method 'newMediaLabel'
        QtMocHelpers::MethodData<QString(Enums::MessageType)>(31, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'newMediaIconName'
        QtMocHelpers::MethodData<QString(Enums::MessageType)>(34, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'label'
        QtMocHelpers::MethodData<QString(Enums::MessageType)>(35, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'iconName'
        QtMocHelpers::MethodData<QString(Enums::MessageType)>(25, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'filterName'
        QtMocHelpers::MethodData<QString(Enums::MessageType)>(36, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'filter'
        QtMocHelpers::MethodData<QString(Enums::MessageType)>(37, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'namedFilter'
        QtMocHelpers::MethodData<QString(Enums::MessageType)>(38, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'mimeTypes'
        QtMocHelpers::MethodData<QList<QMimeType>(Enums::MessageType)>(39, 2, QMC::AccessPublic, 0x80000000 | 40, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'messageType'
        QtMocHelpers::MethodData<Enums::MessageType(const QString &)>(41, 2, QMC::AccessPublic, 0x80000000 | 32, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'messageType'
        QtMocHelpers::MethodData<Enums::MessageType(const QUrl &)>(41, 2, QMC::AccessPublic, 0x80000000 | 32, {{
            { 0x80000000 | 13, 15 },
        }}),
        // Method 'messageType'
        QtMocHelpers::MethodData<Enums::MessageType(const QMimeType &)>(41, 2, QMC::AccessPublic, 0x80000000 | 32, {{
            { 0x80000000 | 23, 22 },
        }}),
        // Method 'mimeTypeName'
        QtMocHelpers::MethodData<QString(const QString &)>(42, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'mimeTypeName'
        QtMocHelpers::MethodData<QString(const QUrl &)>(42, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 13, 15 },
        }}),
        // Method 'hasAlphaChannel'
        QtMocHelpers::MethodData<bool(const QVariant &)>(43, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 44, 45 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MediaUtils, qt_meta_tag_ZN10MediaUtilsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MediaUtils::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MediaUtilsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MediaUtilsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MediaUtilsE_t>.metaTypes,
    nullptr
} };

void MediaUtils::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MediaUtils *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->prettyDuration((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->prettyDuration((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->isHttp((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->isGeoLocation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { QGeoCoordinate _r = _t->locationCoordinate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->localFileAvailable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { QUrl _r = _t->urlFromClipboard();
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->deleteFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->deleteDownloadedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: { QString _r = _t->localFilePath((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QUrl _r = _t->localFileUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->imageValid((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { QMimeType _r = _t->mimeType((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QMimeType*>(_a[0]) = std::move(_r); }  break;
        case 13: { QMimeType _r = _t->mimeType((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QMimeType*>(_a[0]) = std::move(_r); }  break;
        case 14: { QMimeType _r = _t->mimeType((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QMimeType*>(_a[0]) = std::move(_r); }  break;
        case 15: { QString _r = _t->iconName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 16: { QString _r = _t->iconName((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 17: { QUrl _r = _t->newAudioFileUrl();
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 18: { QUrl _r = _t->newImageFileUrl();
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 19: { QUrl _r = _t->newVideoFileUrl();
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 20: { QUrl _r = _t->openFile();
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->openFileInFolder((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 22: { QString _r = _t->newMediaLabel((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { QString _r = _t->newMediaIconName((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 24: { QString _r = _t->label((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 25: { QString _r = _t->iconName((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 26: { QString _r = _t->filterName((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 27: { QString _r = _t->filter((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 28: { QString _r = _t->namedFilter((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 29: { QList<QMimeType> _r = _t->mimeTypes((*reinterpret_cast<std::add_pointer_t<Enums::MessageType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<QMimeType>*>(_a[0]) = std::move(_r); }  break;
        case 30: { Enums::MessageType _r = _t->messageType((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Enums::MessageType*>(_a[0]) = std::move(_r); }  break;
        case 31: { Enums::MessageType _r = _t->messageType((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Enums::MessageType*>(_a[0]) = std::move(_r); }  break;
        case 32: { Enums::MessageType _r = _t->messageType((*reinterpret_cast<std::add_pointer_t<QMimeType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Enums::MessageType*>(_a[0]) = std::move(_r); }  break;
        case 33: { QString _r = _t->mimeTypeName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 34: { QString _r = _t->mimeTypeName((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->hasAlphaChannel((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *MediaUtils::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MediaUtils::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MediaUtilsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MediaUtils::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    return _id;
}
QT_WARNING_POP
