/****************************************************************************
** Meta object code from reading C++ file 'RosterFilterModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/RosterFilterModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RosterFilterModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17RosterFilterModelE_t {};
} // unnamed namespace

template <> constexpr inline auto RosterFilterModel::qt_create_metaobjectdata<qt_meta_tag_ZN17RosterFilterModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RosterFilterModel",
        "displayedTypesChanged",
        "",
        "selectedAccountJidsChanged",
        "selectedGroupsChanged",
        "addDisplayedType",
        "RosterFilterModel::Type",
        "type",
        "removeDisplayedType",
        "resetDisplayedTypes",
        "reorderPinnedItem",
        "oldIndex",
        "newIndex",
        "displayedTypes",
        "Types",
        "selectedAccountJids",
        "selectedGroups",
        "Type",
        "AvailableContact",
        "UnavailableContact",
        "PrivateGroupChat",
        "PublicGroupChat"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'displayedTypesChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedAccountJidsChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedGroupsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'addDisplayedType'
        QtMocHelpers::MethodData<void(RosterFilterModel::Type)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Method 'removeDisplayedType'
        QtMocHelpers::MethodData<void(RosterFilterModel::Type)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Method 'resetDisplayedTypes'
        QtMocHelpers::MethodData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'reorderPinnedItem'
        QtMocHelpers::MethodData<void(int, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'displayedTypes'
        QtMocHelpers::PropertyData<Types>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'selectedAccountJids'
        QtMocHelpers::PropertyData<QList<QString>>(15, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'selectedGroups'
        QtMocHelpers::PropertyData<QList<QString>>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(17, 17, QMC::EnumIsScoped).add({
            {   18, Type::AvailableContact },
            {   19, Type::UnavailableContact },
            {   20, Type::PrivateGroupChat },
            {   21, Type::PublicGroupChat },
        }),
        // flag 'Types'
        QtMocHelpers::EnumData<Types>(14, 17, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {   18, Type::AvailableContact },
            {   19, Type::UnavailableContact },
            {   20, Type::PrivateGroupChat },
            {   21, Type::PublicGroupChat },
        }),
    };
    return QtMocHelpers::metaObjectData<RosterFilterModel, qt_meta_tag_ZN17RosterFilterModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RosterFilterModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17RosterFilterModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17RosterFilterModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17RosterFilterModelE_t>.metaTypes,
    nullptr
} };

void RosterFilterModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RosterFilterModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->displayedTypesChanged(); break;
        case 1: _t->selectedAccountJidsChanged(); break;
        case 2: _t->selectedGroupsChanged(); break;
        case 3: _t->addDisplayedType((*reinterpret_cast<std::add_pointer_t<RosterFilterModel::Type>>(_a[1]))); break;
        case 4: _t->removeDisplayedType((*reinterpret_cast<std::add_pointer_t<RosterFilterModel::Type>>(_a[1]))); break;
        case 5: _t->resetDisplayedTypes(); break;
        case 6: _t->reorderPinnedItem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RosterFilterModel::*)()>(_a, &RosterFilterModel::displayedTypesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterFilterModel::*)()>(_a, &RosterFilterModel::selectedAccountJidsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterFilterModel::*)()>(_a, &RosterFilterModel::selectedGroupsChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::assignFlags<Types>(_v, _t->displayedTypes()); break;
        case 1: *reinterpret_cast<QList<QString>*>(_v) = _t->selectedAccountJids(); break;
        case 2: *reinterpret_cast<QList<QString>*>(_v) = _t->selectedGroups(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSelectedAccountJids(*reinterpret_cast<QList<QString>*>(_v)); break;
        case 2: _t->setSelectedGroups(*reinterpret_cast<QList<QString>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *RosterFilterModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RosterFilterModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17RosterFilterModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int RosterFilterModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void RosterFilterModel::displayedTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void RosterFilterModel::selectedAccountJidsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void RosterFilterModel::selectedGroupsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
