/****************************************************************************
** Meta object code from reading C++ file 'RosterItem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/RosterItem.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RosterItem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10RosterItemE_t {};
} // unnamed namespace

template <> constexpr inline auto RosterItem::qt_create_metaobjectdata<qt_meta_tag_ZN10RosterItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RosterItem",
        "accountJid",
        "jid",
        "name",
        "displayName",
        "sendingPresence",
        "receivingPresence",
        "isNotesChat",
        "isProviderChat",
        "isGroupChat",
        "isPublicGroupChat",
        "isDeletedGroupChat",
        "groups",
        "unreadMessageCount",
        "chatStateSendingEnabled",
        "readMarkerSendingEnabled",
        "notificationRule",
        "RosterItem::NotificationRule",
        "automaticMediaDownloadsRule",
        "RosterItem::AutomaticMediaDownloadsRule",
        "NotificationRule",
        "Account",
        "Never",
        "Mentioned",
        "Always",
        "EffectiveNotificationRule",
        "AutomaticMediaDownloadsRule",
        "GroupChatFlag",
        "Public",
        "Deleted"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accountJid'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'jid'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'displayName'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'sendingPresence'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'receivingPresence'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isNotesChat'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isProviderChat'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isGroupChat'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isPublicGroupChat'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isDeletedGroupChat'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'groups'
        QtMocHelpers::PropertyData<QList<QString>>(12, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'unreadMessageCount'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'chatStateSendingEnabled'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'readMarkerSendingEnabled'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'notificationRule'
        QtMocHelpers::PropertyData<RosterItem::NotificationRule>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'automaticMediaDownloadsRule'
        QtMocHelpers::PropertyData<RosterItem::AutomaticMediaDownloadsRule>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'NotificationRule'
        QtMocHelpers::EnumData<enum NotificationRule>(20, 20, QMC::EnumIsScoped).add({
            {   21, NotificationRule::Account },
            {   22, NotificationRule::Never },
            {   23, NotificationRule::Mentioned },
            {   24, NotificationRule::Always },
        }),
        // enum 'EffectiveNotificationRule'
        QtMocHelpers::EnumData<enum EffectiveNotificationRule>(25, 25, QMC::EnumIsScoped).add({
            {   22, EffectiveNotificationRule::Never },
            {   23, EffectiveNotificationRule::Mentioned },
            {   24, EffectiveNotificationRule::Always },
        }),
        // enum 'AutomaticMediaDownloadsRule'
        QtMocHelpers::EnumData<enum AutomaticMediaDownloadsRule>(26, 26, QMC::EnumIsScoped).add({
            {   21, AutomaticMediaDownloadsRule::Account },
            {   22, AutomaticMediaDownloadsRule::Never },
            {   24, AutomaticMediaDownloadsRule::Always },
        }),
        // enum 'GroupChatFlag'
        QtMocHelpers::EnumData<enum GroupChatFlag>(27, 27, QMC::EnumIsScoped).add({
            {   28, GroupChatFlag::Public },
            {   29, GroupChatFlag::Deleted },
        }),
    };
    return QtMocHelpers::metaObjectData<RosterItem, qt_meta_tag_ZN10RosterItemE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RosterItem::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RosterItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RosterItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10RosterItemE_t>.metaTypes,
    nullptr
} };

void RosterItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<RosterItem *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 16:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< RosterItem::AutomaticMediaDownloadsRule >(); break;
        case 15:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< RosterItem::NotificationRule >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->accountJid; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->jid; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->name; break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->displayName(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isSendingPresence(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isReceivingPresence(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isNotesChat(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isProviderChat(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isGroupChat(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isPublicGroupChat(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isDeletedGroupChat(); break;
        case 11: *reinterpret_cast<QList<QString>*>(_v) = _t->groups; break;
        case 12: *reinterpret_cast<int*>(_v) = _t->unreadMessageCount; break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->chatStateSendingEnabled; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->readMarkerSendingEnabled; break;
        case 15: *reinterpret_cast<RosterItem::NotificationRule*>(_v) = _t->notificationRule; break;
        case 16: *reinterpret_cast<RosterItem::AutomaticMediaDownloadsRule*>(_v) = _t->automaticMediaDownloadsRule; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->accountJid, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->jid, *reinterpret_cast<QString*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->name, *reinterpret_cast<QString*>(_v)); break;
        case 11: QtMocHelpers::setProperty(_t->groups, *reinterpret_cast<QList<QString>*>(_v)); break;
        case 12: QtMocHelpers::setProperty(_t->unreadMessageCount, *reinterpret_cast<int*>(_v)); break;
        case 13: QtMocHelpers::setProperty(_t->chatStateSendingEnabled, *reinterpret_cast<bool*>(_v)); break;
        case 14: QtMocHelpers::setProperty(_t->readMarkerSendingEnabled, *reinterpret_cast<bool*>(_v)); break;
        case 15: QtMocHelpers::setProperty(_t->notificationRule, *reinterpret_cast<RosterItem::NotificationRule*>(_v)); break;
        case 16: QtMocHelpers::setProperty(_t->automaticMediaDownloadsRule, *reinterpret_cast<RosterItem::AutomaticMediaDownloadsRule*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
