/****************************************************************************
** Meta object code from reading C++ file 'RegistrationController.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/RegistrationController.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RegistrationController.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22RegistrationControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto RegistrationController::qt_create_metaobjectdata<qt_meta_tag_ZN22RegistrationControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RegistrationController",
        "registrationFormReceived",
        "",
        "DataFormModel*",
        "dataFormModel",
        "registrationOutOfBandUrlReceived",
        "QUrl",
        "outOfBandUrl",
        "registrationFailed",
        "RegistrationError",
        "error",
        "errorMessage",
        "passwordChangeSucceeded",
        "passwordChangeFailed",
        "accountDeletionFromClientAndServerFailed",
        "requestRegistrationForm",
        "sendRegistrationForm",
        "abortRegistration",
        "changePassword",
        "newPassword",
        "deleteAccountFromClient",
        "deleteAccountFromClientAndServer",
        "UnknownError",
        "InBandRegistrationNotSupported",
        "UsernameConflict",
        "PasswordTooWeak",
        "CaptchaVerificationFailed",
        "RequiredInformationMissing",
        "TemporarilyBlocked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'registrationFormReceived'
        QtMocHelpers::SignalData<void(DataFormModel *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'registrationOutOfBandUrlReceived'
        QtMocHelpers::SignalData<void(const QUrl &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'registrationFailed'
        QtMocHelpers::SignalData<void(enum RegistrationError, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'passwordChangeSucceeded'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'passwordChangeFailed'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'accountDeletionFromClientAndServerFailed'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Method 'requestRegistrationForm'
        QtMocHelpers::MethodData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'sendRegistrationForm'
        QtMocHelpers::MethodData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'abortRegistration'
        QtMocHelpers::MethodData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'changePassword'
        QtMocHelpers::MethodData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Method 'deleteAccountFromClient'
        QtMocHelpers::MethodData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'deleteAccountFromClientAndServer'
        QtMocHelpers::MethodData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'RegistrationError'
        QtMocHelpers::EnumData<enum RegistrationError>(9, 9, QMC::EnumFlags{}).add({
            {   22, RegistrationError::UnknownError },
            {   23, RegistrationError::InBandRegistrationNotSupported },
            {   24, RegistrationError::UsernameConflict },
            {   25, RegistrationError::PasswordTooWeak },
            {   26, RegistrationError::CaptchaVerificationFailed },
            {   27, RegistrationError::RequiredInformationMissing },
            {   28, RegistrationError::TemporarilyBlocked },
        }),
    };
    return QtMocHelpers::metaObjectData<RegistrationController, qt_meta_tag_ZN22RegistrationControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RegistrationController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22RegistrationControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22RegistrationControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22RegistrationControllerE_t>.metaTypes,
    nullptr
} };

void RegistrationController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RegistrationController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->registrationFormReceived((*reinterpret_cast<std::add_pointer_t<DataFormModel*>>(_a[1]))); break;
        case 1: _t->registrationOutOfBandUrlReceived((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 2: _t->registrationFailed((*reinterpret_cast<std::add_pointer_t<enum RegistrationError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->passwordChangeSucceeded(); break;
        case 4: _t->passwordChangeFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->accountDeletionFromClientAndServerFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->requestRegistrationForm(); break;
        case 7: _t->sendRegistrationForm(); break;
        case 8: _t->abortRegistration(); break;
        case 9: _t->changePassword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->deleteAccountFromClient(); break;
        case 11: _t->deleteAccountFromClientAndServer(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RegistrationController::*)(DataFormModel * )>(_a, &RegistrationController::registrationFormReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RegistrationController::*)(const QUrl & )>(_a, &RegistrationController::registrationOutOfBandUrlReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RegistrationController::*)(RegistrationError , const QString & )>(_a, &RegistrationController::registrationFailed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RegistrationController::*)()>(_a, &RegistrationController::passwordChangeSucceeded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RegistrationController::*)(const QString & )>(_a, &RegistrationController::passwordChangeFailed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (RegistrationController::*)(const QString & )>(_a, &RegistrationController::accountDeletionFromClientAndServerFailed, 5))
            return;
    }
}

const QMetaObject *RegistrationController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RegistrationController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22RegistrationControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int RegistrationController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void RegistrationController::registrationFormReceived(DataFormModel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void RegistrationController::registrationOutOfBandUrlReceived(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void RegistrationController::registrationFailed(RegistrationError _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void RegistrationController::passwordChangeSucceeded()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void RegistrationController::passwordChangeFailed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void RegistrationController::accountDeletionFromClientAndServerFailed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
