/****************************************************************************
** Meta object code from reading C++ file 'EncryptionWatcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/EncryptionWatcher.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EncryptionWatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17EncryptionWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto EncryptionWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN17EncryptionWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EncryptionWatcher",
        "accountJidChanged",
        "",
        "jidsChanged",
        "hasDistrustedDevicesChanged",
        "hasUsableDevicesChanged",
        "hasAuthenticatableDevicesChanged",
        "hasAuthenticatableDistrustedDevicesChanged",
        "encryptionController",
        "EncryptionController*",
        "accountJid",
        "jids",
        "hasDistrustedDevices",
        "hasUsableDevices",
        "hasAuthenticatableDevices",
        "hasAuthenticatableDistrustedDevices"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountJidChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'jidsChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasDistrustedDevicesChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasUsableDevicesChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasAuthenticatableDevicesChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasAuthenticatableDistrustedDevicesChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'encryptionController'
        QtMocHelpers::PropertyData<EncryptionController*>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'accountJid'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'jids'
        QtMocHelpers::PropertyData<QList<QString>>(11, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'hasDistrustedDevices'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'hasUsableDevices'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'hasAuthenticatableDevices'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'hasAuthenticatableDistrustedDevices'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EncryptionWatcher, qt_meta_tag_ZN17EncryptionWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EncryptionWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17EncryptionWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17EncryptionWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17EncryptionWatcherE_t>.metaTypes,
    nullptr
} };

void EncryptionWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EncryptionWatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountJidChanged(); break;
        case 1: _t->jidsChanged(); break;
        case 2: _t->hasDistrustedDevicesChanged(); break;
        case 3: _t->hasUsableDevicesChanged(); break;
        case 4: _t->hasAuthenticatableDevicesChanged(); break;
        case 5: _t->hasAuthenticatableDistrustedDevicesChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EncryptionWatcher::*)()>(_a, &EncryptionWatcher::accountJidChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EncryptionWatcher::*)()>(_a, &EncryptionWatcher::jidsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EncryptionWatcher::*)()>(_a, &EncryptionWatcher::hasDistrustedDevicesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (EncryptionWatcher::*)()>(_a, &EncryptionWatcher::hasUsableDevicesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (EncryptionWatcher::*)()>(_a, &EncryptionWatcher::hasAuthenticatableDevicesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (EncryptionWatcher::*)()>(_a, &EncryptionWatcher::hasAuthenticatableDistrustedDevicesChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<EncryptionController**>(_v) = _t->m_encryptionController; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->accountJid(); break;
        case 2: *reinterpret_cast<QList<QString>*>(_v) = _t->jids(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasDistrustedDevices(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hasUsableDevices(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->hasAuthenticatableDevices(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->hasAuthenticatableDistrustedDevices(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEncryptionController(*reinterpret_cast<EncryptionController**>(_v)); break;
        case 1: _t->setAccountJid(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setJids(*reinterpret_cast<QList<QString>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *EncryptionWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EncryptionWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17EncryptionWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int EncryptionWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void EncryptionWatcher::accountJidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void EncryptionWatcher::jidsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void EncryptionWatcher::hasDistrustedDevicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void EncryptionWatcher::hasUsableDevicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void EncryptionWatcher::hasAuthenticatableDevicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void EncryptionWatcher::hasAuthenticatableDistrustedDevicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
