/****************************************************************************
** Meta object code from reading C++ file 'RosterModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/RosterModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RosterModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11RosterModelE_t {};
} // unnamed namespace

template <> constexpr inline auto RosterModel::qt_create_metaobjectdata<qt_meta_tag_ZN11RosterModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RosterModel",
        "itemsFetched",
        "",
        "QList<RosterItem>",
        "items",
        "itemAdded",
        "RosterItem",
        "item",
        "itemRemoved",
        "accountJid",
        "jid",
        "itemsRemoved",
        "groupsChanged",
        "hasItem",
        "setItemEncryption",
        "Encryption::Enum",
        "encryption",
        "pinItem",
        "unpinItem",
        "reorderPinnedItem",
        "oldIndex",
        "newIndex",
        "toggleSelected",
        "resetSelected",
        "groups",
        "RosterItemRoles",
        "AccountRole",
        "JidRole",
        "NameRole",
        "GroupsRole",
        "IsNotesChatRole",
        "IsProviderChatRole",
        "IsGroupChatRole",
        "IsPublicGroupChatRole",
        "IsDeletedGroupChatRole",
        "LastMessageDateTimeRole",
        "UnreadMessageCountRole",
        "MarkedMessageCountRole",
        "LastMessageRole",
        "LastMessageIsDraftRole",
        "LastMessageIsOwnRole",
        "LastMessageGroupChatSenderNameRole",
        "PinnedRole",
        "SelectedRole",
        "EffectiveNotificationRuleRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemsFetched'
        QtMocHelpers::SignalData<void(const QList<RosterItem> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'itemAdded'
        QtMocHelpers::SignalData<void(const RosterItem &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'itemRemoved'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'itemsRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'groupsChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'hasItem'
        QtMocHelpers::MethodData<bool(const QString &, const QString &) const>(13, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Method 'setItemEncryption'
        QtMocHelpers::MethodData<void(const QString &, Encryption::Enum)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { 0x80000000 | 15, 16 },
        }}),
        // Method 'pinItem'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Method 'unpinItem'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Method 'reorderPinnedItem'
        QtMocHelpers::MethodData<void(int, int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { QMetaType::Int, 21 },
        }}),
        // Method 'toggleSelected'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Method 'resetSelected'
        QtMocHelpers::MethodData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'groups'
        QtMocHelpers::PropertyData<QStringList>(24, QMetaType::QStringList, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'RosterItemRoles'
        QtMocHelpers::EnumData<enum RosterItemRoles>(25, 25, QMC::EnumFlags{}).add({
            {   26, RosterItemRoles::AccountRole },
            {   27, RosterItemRoles::JidRole },
            {   28, RosterItemRoles::NameRole },
            {   29, RosterItemRoles::GroupsRole },
            {   30, RosterItemRoles::IsNotesChatRole },
            {   31, RosterItemRoles::IsProviderChatRole },
            {   32, RosterItemRoles::IsGroupChatRole },
            {   33, RosterItemRoles::IsPublicGroupChatRole },
            {   34, RosterItemRoles::IsDeletedGroupChatRole },
            {   35, RosterItemRoles::LastMessageDateTimeRole },
            {   36, RosterItemRoles::UnreadMessageCountRole },
            {   37, RosterItemRoles::MarkedMessageCountRole },
            {   38, RosterItemRoles::LastMessageRole },
            {   39, RosterItemRoles::LastMessageIsDraftRole },
            {   40, RosterItemRoles::LastMessageIsOwnRole },
            {   41, RosterItemRoles::LastMessageGroupChatSenderNameRole },
            {   42, RosterItemRoles::PinnedRole },
            {   43, RosterItemRoles::SelectedRole },
            {   44, RosterItemRoles::EffectiveNotificationRuleRole },
        }),
    };
    return QtMocHelpers::metaObjectData<RosterModel, qt_meta_tag_ZN11RosterModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RosterModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11RosterModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11RosterModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11RosterModelE_t>.metaTypes,
    nullptr
} };

void RosterModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RosterModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemsFetched((*reinterpret_cast<std::add_pointer_t<QList<RosterItem>>>(_a[1]))); break;
        case 1: _t->itemAdded((*reinterpret_cast<std::add_pointer_t<RosterItem>>(_a[1]))); break;
        case 2: _t->itemRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->itemsRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->groupsChanged(); break;
        case 5: { bool _r = _t->hasItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setItemEncryption((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Encryption::Enum>>(_a[2]))); break;
        case 7: _t->pinItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->unpinItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->reorderPinnedItem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->toggleSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->resetSelected(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<RosterItem> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< RosterItem >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RosterModel::*)(const QList<RosterItem> & )>(_a, &RosterModel::itemsFetched, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterModel::*)(const RosterItem & )>(_a, &RosterModel::itemAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterModel::*)(const QString & , const QString & )>(_a, &RosterModel::itemRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterModel::*)(const QString & )>(_a, &RosterModel::itemsRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RosterModel::*)()>(_a, &RosterModel::groupsChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->groups(); break;
        default: break;
        }
    }
}

const QMetaObject *RosterModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RosterModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11RosterModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int RosterModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void RosterModel::itemsFetched(const QList<RosterItem> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void RosterModel::itemAdded(const RosterItem & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void RosterModel::itemRemoved(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void RosterModel::itemsRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void RosterModel::groupsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
