/****************************************************************************
** Meta object code from reading C++ file 'FileProxyModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/FileProxyModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FileProxyModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14FileProxyModelE_t {};
} // unnamed namespace

template <> constexpr inline auto FileProxyModel::qt_create_metaobjectdata<qt_meta_tag_ZN14FileProxyModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileProxyModel",
        "modeChanged",
        "",
        "locallyAvailableOnlyChanged",
        "attachmentAudioOnlyChanged",
        "checkedCountChanged",
        "filesDeleted",
        "files",
        "errors",
        "rowCountChanged",
        "checkAll",
        "clearChecked",
        "deleteChecked",
        "fileIndex",
        "QModelIndex",
        "fileId",
        "mode",
        "FileProxyModel::Mode",
        "checkedCount",
        "rowCount",
        "locallyAvailableOnly",
        "attachmentAudioOnly",
        "Mode",
        "All",
        "Images",
        "Videos",
        "Other"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'locallyAvailableOnlyChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'attachmentAudioOnlyChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'checkedCountChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filesDeleted'
        QtMocHelpers::SignalData<void(const QStringList &, const QStringList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 7 }, { QMetaType::QStringList, 8 },
        }}),
        // Signal 'rowCountChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkAll'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearChecked'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteChecked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'fileIndex'
        QtMocHelpers::MethodData<QModelIndex(const QString &) const>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'mode'
        QtMocHelpers::PropertyData<FileProxyModel::Mode>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'checkedCount'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags, 3),
        // property 'rowCount'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags, 5),
        // property 'locallyAvailableOnly'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'attachmentAudioOnly'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Mode'
        QtMocHelpers::EnumData<enum Mode>(22, 22, QMC::EnumIsScoped).add({
            {   23, Mode::All },
            {   24, Mode::Images },
            {   25, Mode::Videos },
            {   26, Mode::Other },
        }),
    };
    return QtMocHelpers::metaObjectData<FileProxyModel, qt_meta_tag_ZN14FileProxyModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileProxyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14FileProxyModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14FileProxyModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14FileProxyModelE_t>.metaTypes,
    nullptr
} };

void FileProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileProxyModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->modeChanged(); break;
        case 1: _t->locallyAvailableOnlyChanged(); break;
        case 2: _t->attachmentAudioOnlyChanged(); break;
        case 3: _t->checkedCountChanged(); break;
        case 4: _t->filesDeleted((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 5: _t->rowCountChanged(); break;
        case 6: _t->checkAll(); break;
        case 7: _t->clearChecked(); break;
        case 8: _t->deleteChecked(); break;
        case 9: { QModelIndex _r = _t->fileIndex((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileProxyModel::*)()>(_a, &FileProxyModel::modeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileProxyModel::*)()>(_a, &FileProxyModel::locallyAvailableOnlyChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileProxyModel::*)()>(_a, &FileProxyModel::attachmentAudioOnlyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileProxyModel::*)()>(_a, &FileProxyModel::checkedCountChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileProxyModel::*)(const QStringList & , const QStringList & )>(_a, &FileProxyModel::filesDeleted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileProxyModel::*)()>(_a, &FileProxyModel::rowCountChanged, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FileProxyModel::Mode >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<FileProxyModel::Mode*>(_v) = _t->mode(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->checkedCount(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->rowCount(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->locallyAvailableOnly(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->attachmentAudioOnly(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMode(*reinterpret_cast<FileProxyModel::Mode*>(_v)); break;
        case 3: _t->setLocallyAvailableOnly(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setAttachmentAudioOnly(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *FileProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14FileProxyModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int FileProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void FileProxyModel::modeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FileProxyModel::locallyAvailableOnlyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FileProxyModel::attachmentAudioOnlyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void FileProxyModel::checkedCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void FileProxyModel::filesDeleted(const QStringList & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void FileProxyModel::rowCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
