/****************************************************************************
** Meta object code from reading C++ file 'ImageProvider.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/ImageProvider.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ImageProvider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ImageProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto ImageProvider::qt_create_metaobjectdata<qt_meta_tag_ZN13ImageProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ImageProvider",
        "screenDevicePixelRatioChanged",
        "",
        "imageAlphaChannelChanged",
        "QUrl",
        "source",
        "hasAlphaChannel",
        "sourceCopiedToClipboard",
        "QSize",
        "size",
        "copySourceToClipboard",
        "generatedFileImageUrl",
        "File",
        "file",
        "generatedQrCodeImageUrl",
        "text",
        "screenDevicePixelRatio"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'screenDevicePixelRatioChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'imageAlphaChannelChanged'
        QtMocHelpers::SignalData<void(const QUrl &, bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Signal 'sourceCopiedToClipboard'
        QtMocHelpers::SignalData<void(const QUrl &, const QSize &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 8, 9 },
        }}),
        // Method 'copySourceToClipboard'
        QtMocHelpers::MethodData<void(const QUrl &, const QSize &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 8, 9 },
        }}),
        // Method 'generatedFileImageUrl'
        QtMocHelpers::MethodData<QUrl(const File &)>(11, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'generatedQrCodeImageUrl'
        QtMocHelpers::MethodData<QUrl(const QString &)>(14, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'screenDevicePixelRatio'
        QtMocHelpers::PropertyData<qreal>(16, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ImageProvider, qt_meta_tag_ZN13ImageProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ImageProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickAsyncImageProvider::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ImageProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ImageProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ImageProviderE_t>.metaTypes,
    nullptr
} };

void ImageProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ImageProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->screenDevicePixelRatioChanged(); break;
        case 1: _t->imageAlphaChannelChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->sourceCopiedToClipboard((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2]))); break;
        case 3: _t->copySourceToClipboard((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2]))); break;
        case 4: { QUrl _r = _t->generatedFileImageUrl((*reinterpret_cast<std::add_pointer_t<File>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 5: { QUrl _r = _t->generatedQrCodeImageUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ImageProvider::*)()>(_a, &ImageProvider::screenDevicePixelRatioChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImageProvider::*)(const QUrl & , bool )>(_a, &ImageProvider::imageAlphaChannelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImageProvider::*)(const QUrl & , const QSize & )>(_a, &ImageProvider::sourceCopiedToClipboard, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->screenDevicePixelRatio(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setScreenDevicePixelRatio(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ImageProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImageProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ImageProviderE_t>.strings))
        return static_cast<void*>(this);
    return QQuickAsyncImageProvider::qt_metacast(_clname);
}

int ImageProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickAsyncImageProvider::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void ImageProvider::screenDevicePixelRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ImageProvider::imageAlphaChannelChanged(const QUrl & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void ImageProvider::sourceCopiedToClipboard(const QUrl & _t1, const QSize & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
