/****************************************************************************
** Meta object code from reading C++ file 'Enums.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/Enums.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Enums.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5EnumsE_t {};
} // unnamed namespace

template <> constexpr inline auto Enums::qt_create_metaobjectdata<qt_meta_tag_ZN5EnumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Enums",
        "ConnectionState",
        "StateDisconnected",
        "StateConnecting",
        "StateConnected",
        "MessageType",
        "MessageUnknown",
        "MessageText",
        "MessageFile",
        "MessageImage",
        "MessageVideo",
        "MessageAudio",
        "MessageDocument",
        "MessageGeoLocation",
        "DeliveryState",
        "Pending",
        "Sent",
        "Delivered",
        "Error",
        "Draft"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ConnectionState'
        QtMocHelpers::EnumData<ConnectionState>(1, 1, QMC::EnumIsScoped).add({
            {    2, ConnectionState::StateDisconnected },
            {    3, ConnectionState::StateConnecting },
            {    4, ConnectionState::StateConnected },
        }),
        // enum 'MessageType'
        QtMocHelpers::EnumData<MessageType>(5, 5, QMC::EnumIsScoped).add({
            {    6, MessageType::MessageUnknown },
            {    7, MessageType::MessageText },
            {    8, MessageType::MessageFile },
            {    9, MessageType::MessageImage },
            {   10, MessageType::MessageVideo },
            {   11, MessageType::MessageAudio },
            {   12, MessageType::MessageDocument },
            {   13, MessageType::MessageGeoLocation },
        }),
        // enum 'DeliveryState'
        QtMocHelpers::EnumData<DeliveryState>(14, 14, QMC::EnumIsScoped).add({
            {   15, DeliveryState::Pending },
            {   16, DeliveryState::Sent },
            {   17, DeliveryState::Delivered },
            {   18, DeliveryState::Error },
            {   19, DeliveryState::Draft },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN5EnumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN5EnumsE =
    Enums::qt_create_metaobjectdata<qt_meta_tag_ZN5EnumsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN5EnumsE =
    qt_staticMetaObjectContent_ZN5EnumsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN5EnumsE =
    qt_staticMetaObjectContent_ZN5EnumsE.relocatingData;

Q_CONSTINIT const QMetaObject Enums::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN5EnumsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN5EnumsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN5EnumsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
