/****************************************************************************
** Meta object code from reading C++ file 'ProviderFilterModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/ProviderFilterModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ProviderFilterModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19ProviderFilterModelE_t {};
} // unnamed namespace

template <> constexpr inline auto ProviderFilterModel::qt_create_metaobjectdata<qt_meta_tag_ZN19ProviderFilterModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ProviderFilterModel",
        "supportsInBandRegistrationOnlyChanged",
        "",
        "flagChanged",
        "mininumBusFactorChanged",
        "organizationChanged",
        "supportsPasswordResetOnlyChanged",
        "hostedGreenOnlyChanged",
        "minimumAvailabilityChanged",
        "minimumHttpUploadFileSizeChanged",
        "minimumHttpUploadTotalSizeChanged",
        "minimumHttpUploadStorageDurationChanged",
        "minimumMessageStorageDurationChanged",
        "freeOfChargeOnlyChanged",
        "supportsInBandRegistrationOnly",
        "flag",
        "mininumBusFactor",
        "organization",
        "supportsPasswordResetOnly",
        "hostedGreenOnly",
        "availabilities",
        "QList<int>",
        "minimumAvailability",
        "httpUploadFileSizes",
        "minimumHttpUploadFileSize",
        "httpUploadTotalSizes",
        "minimumHttpUploadTotalSize",
        "httpUploadStorageDurations",
        "minimumHttpUploadStorageDuration",
        "messageStorageDurations",
        "minimumMessageStorageDuration",
        "freeOfChargeOnly"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'supportsInBandRegistrationOnlyChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'flagChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mininumBusFactorChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'organizationChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'supportsPasswordResetOnlyChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hostedGreenOnlyChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumAvailabilityChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumHttpUploadFileSizeChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumHttpUploadTotalSizeChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumHttpUploadStorageDurationChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumMessageStorageDurationChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'freeOfChargeOnlyChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'supportsInBandRegistrationOnly'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'flag'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'mininumBusFactor'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'organization'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'supportsPasswordResetOnly'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'hostedGreenOnly'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'availabilities'
        QtMocHelpers::PropertyData<QList<int>>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'minimumAvailability'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'httpUploadFileSizes'
        QtMocHelpers::PropertyData<QList<int>>(23, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'minimumHttpUploadFileSize'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'httpUploadTotalSizes'
        QtMocHelpers::PropertyData<QList<int>>(25, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'minimumHttpUploadTotalSize'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'httpUploadStorageDurations'
        QtMocHelpers::PropertyData<QList<int>>(27, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'minimumHttpUploadStorageDuration'
        QtMocHelpers::PropertyData<int>(28, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'messageStorageDurations'
        QtMocHelpers::PropertyData<QList<int>>(29, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'minimumMessageStorageDuration'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'freeOfChargeOnly'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProviderFilterModel, qt_meta_tag_ZN19ProviderFilterModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ProviderFilterModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ProviderFilterModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ProviderFilterModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19ProviderFilterModelE_t>.metaTypes,
    nullptr
} };

void ProviderFilterModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProviderFilterModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->supportsInBandRegistrationOnlyChanged(); break;
        case 1: _t->flagChanged(); break;
        case 2: _t->mininumBusFactorChanged(); break;
        case 3: _t->organizationChanged(); break;
        case 4: _t->supportsPasswordResetOnlyChanged(); break;
        case 5: _t->hostedGreenOnlyChanged(); break;
        case 6: _t->minimumAvailabilityChanged(); break;
        case 7: _t->minimumHttpUploadFileSizeChanged(); break;
        case 8: _t->minimumHttpUploadTotalSizeChanged(); break;
        case 9: _t->minimumHttpUploadStorageDurationChanged(); break;
        case 10: _t->minimumMessageStorageDurationChanged(); break;
        case 11: _t->freeOfChargeOnlyChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::supportsInBandRegistrationOnlyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::flagChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::mininumBusFactorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::organizationChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::supportsPasswordResetOnlyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::hostedGreenOnlyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::minimumAvailabilityChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::minimumHttpUploadFileSizeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::minimumHttpUploadTotalSizeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::minimumHttpUploadStorageDurationChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::minimumMessageStorageDurationChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProviderFilterModel::*)()>(_a, &ProviderFilterModel::freeOfChargeOnlyChanged, 11))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 14:
        case 12:
        case 10:
        case 8:
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->m_supportsInBandRegistrationOnly; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->m_flag; break;
        case 2: *reinterpret_cast<int*>(_v) = _t->m_mininumBusFactor; break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->m_organization; break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->m_supportsPasswordResetOnly; break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->m_hostedGreenOnly; break;
        case 6: *reinterpret_cast<QList<int>*>(_v) = _t->availabilities(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->m_minimumAvailability; break;
        case 8: *reinterpret_cast<QList<int>*>(_v) = _t->httpUploadFileSizes(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->m_minimumHttpUploadFileSize; break;
        case 10: *reinterpret_cast<QList<int>*>(_v) = _t->httpUploadTotalSizes(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->m_minimumHttpUploadTotalSize; break;
        case 12: *reinterpret_cast<QList<int>*>(_v) = _t->httpUploadStorageDurations(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->m_minimumHttpUploadStorageDuration; break;
        case 14: *reinterpret_cast<QList<int>*>(_v) = _t->messageStorageDurations(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->m_minimumMessageStorageDuration; break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->m_freeOfChargeOnly; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSupportsInBandRegistrationOnly(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlag(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setMininumBusFactor(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setOrganization(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setSupportsPasswordResetOnly(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setHostedGreenOnly(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setMinimumAvailability(*reinterpret_cast<int*>(_v)); break;
        case 9: _t->setMinimumHttpUploadFileSize(*reinterpret_cast<int*>(_v)); break;
        case 11: _t->setMinimumHttpUploadTotalSize(*reinterpret_cast<int*>(_v)); break;
        case 13: _t->setMinimumHttpUploadStorageDuration(*reinterpret_cast<int*>(_v)); break;
        case 15: _t->setMinimumMessageStorageDuration(*reinterpret_cast<int*>(_v)); break;
        case 16: _t->setFreeOfChargeOnly(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ProviderFilterModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProviderFilterModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ProviderFilterModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int ProviderFilterModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void ProviderFilterModel::supportsInBandRegistrationOnlyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ProviderFilterModel::flagChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ProviderFilterModel::mininumBusFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ProviderFilterModel::organizationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ProviderFilterModel::supportsPasswordResetOnlyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ProviderFilterModel::hostedGreenOnlyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ProviderFilterModel::minimumAvailabilityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void ProviderFilterModel::minimumHttpUploadFileSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ProviderFilterModel::minimumHttpUploadTotalSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void ProviderFilterModel::minimumHttpUploadStorageDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void ProviderFilterModel::minimumMessageStorageDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void ProviderFilterModel::freeOfChargeOnlyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
