/****************************************************************************
** Meta object code from reading C++ file 'Provider.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/Provider.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Provider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8ProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto Provider::qt_create_metaobjectdata<qt_meta_tag_ZN8ProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Provider",
        "chosenWebsite",
        "QUrl",
        "chosenChatSupport",
        "chosenGroupChatSupport"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'chosenWebsite'
        QtMocHelpers::PropertyData<QUrl>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'chosenChatSupport'
        QtMocHelpers::PropertyData<QList<QString>>(3, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'chosenGroupChatSupport'
        QtMocHelpers::PropertyData<QList<QString>>(4, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Provider, qt_meta_tag_ZN8ProviderE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Provider::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8ProviderE_t>.metaTypes,
    nullptr
} };

void Provider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Provider *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->chosenWebsite(); break;
        case 1: *reinterpret_cast<QList<QString>*>(_v) = _t->chosenChatSupport(); break;
        case 2: *reinterpret_cast<QList<QString>*>(_v) = _t->chosenGroupChatSupport(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
