/****************************************************************************
** Meta object code from reading C++ file 'GroupChatController.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/GroupChatController.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'GroupChatController.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19GroupChatControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto GroupChatController::qt_create_metaobjectdata<qt_meta_tag_ZN19GroupChatControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GroupChatController",
        "busyChanged",
        "",
        "groupChatParticipationSupportedChanged",
        "groupChatServicesChanged",
        "privateGroupChatCreationFailed",
        "serviceJid",
        "errorMessage",
        "publicGroupChatCreationFailed",
        "groupChatJid",
        "groupChatCreated",
        "groupChatJoined",
        "groupChatJoiningFailed",
        "contactInvitedToGroupChat",
        "inviteeJid",
        "groupChatInviteeSelectionNeeded",
        "groupChatLeft",
        "groupChatLeavingFailed",
        "groupChatDeleted",
        "groupChatDeletionFailed",
        "groupChatMadePrivate",
        "groupChatMadePublic",
        "userDisallowedOrUnbanned",
        "GroupChatUser",
        "user",
        "userAllowedOrBanned",
        "participantReceived",
        "participant",
        "participantLeft",
        "groupChatServiceJids",
        "createPrivateGroupChat",
        "createPublicGroupChat",
        "renameGroupChat",
        "name",
        "joinGroupChat",
        "nickname",
        "inviteContactToGroupChat",
        "contactJid",
        "groupChatPublic",
        "leaveGroupChat",
        "deleteGroupChat",
        "banUser",
        "userJid",
        "busy",
        "groupChatParticipationSupported",
        "groupChatCreationSupported"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'busyChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'groupChatParticipationSupportedChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'groupChatServicesChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'privateGroupChatCreationFailed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'publicGroupChatCreationFailed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'groupChatCreated'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'groupChatJoined'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'groupChatJoiningFailed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'contactInvitedToGroupChat'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 14 },
        }}),
        // Signal 'groupChatInviteeSelectionNeeded'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'groupChatLeft'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'groupChatLeavingFailed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'groupChatDeleted'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'groupChatDeletionFailed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'groupChatMadePrivate'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'groupChatMadePublic'
        QtMocHelpers::SignalData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'userDisallowedOrUnbanned'
        QtMocHelpers::SignalData<void(const GroupChatUser &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'userAllowedOrBanned'
        QtMocHelpers::SignalData<void(const GroupChatUser &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Signal 'participantReceived'
        QtMocHelpers::SignalData<void(const GroupChatUser &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 27 },
        }}),
        // Signal 'participantLeft'
        QtMocHelpers::SignalData<void(const GroupChatUser &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 27 },
        }}),
        // Method 'groupChatServiceJids'
        QtMocHelpers::MethodData<QStringList() const>(29, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'createPrivateGroupChat'
        QtMocHelpers::MethodData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'createPublicGroupChat'
        QtMocHelpers::MethodData<void(const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Method 'renameGroupChat'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 33 },
        }}),
        // Method 'joinGroupChat'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 35 },
        }}),
        // Method 'inviteContactToGroupChat'
        QtMocHelpers::MethodData<void(const QString &, const QString &, bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 37 }, { QMetaType::Bool, 38 },
        }}),
        // Method 'leaveGroupChat'
        QtMocHelpers::MethodData<void(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Method 'deleteGroupChat'
        QtMocHelpers::MethodData<void(const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Method 'banUser'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 42 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'busy'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'groupChatParticipationSupported'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'groupChatCreationSupported'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GroupChatController, qt_meta_tag_ZN19GroupChatControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GroupChatController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19GroupChatControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19GroupChatControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19GroupChatControllerE_t>.metaTypes,
    nullptr
} };

void GroupChatController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GroupChatController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->busyChanged(); break;
        case 1: _t->groupChatParticipationSupportedChanged(); break;
        case 2: _t->groupChatServicesChanged(); break;
        case 3: _t->privateGroupChatCreationFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->publicGroupChatCreationFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->groupChatCreated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->groupChatJoined((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->groupChatJoiningFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->contactInvitedToGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->groupChatInviteeSelectionNeeded(); break;
        case 10: _t->groupChatLeft((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->groupChatLeavingFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->groupChatDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->groupChatDeletionFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->groupChatMadePrivate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->groupChatMadePublic((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->userDisallowedOrUnbanned((*reinterpret_cast<std::add_pointer_t<GroupChatUser>>(_a[1]))); break;
        case 17: _t->userAllowedOrBanned((*reinterpret_cast<std::add_pointer_t<GroupChatUser>>(_a[1]))); break;
        case 18: _t->participantReceived((*reinterpret_cast<std::add_pointer_t<GroupChatUser>>(_a[1]))); break;
        case 19: _t->participantLeft((*reinterpret_cast<std::add_pointer_t<GroupChatUser>>(_a[1]))); break;
        case 20: { QStringList _r = _t->groupChatServiceJids();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->createPrivateGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->createPublicGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->renameGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 24: _t->joinGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 25: _t->inviteContactToGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 26: _t->leaveGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->deleteGroupChat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->banUser((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)()>(_a, &GroupChatController::busyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)()>(_a, &GroupChatController::groupChatParticipationSupportedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)()>(_a, &GroupChatController::groupChatServicesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & , const QString & )>(_a, &GroupChatController::privateGroupChatCreationFailed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & , const QString & )>(_a, &GroupChatController::publicGroupChatCreationFailed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & )>(_a, &GroupChatController::groupChatCreated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & )>(_a, &GroupChatController::groupChatJoined, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & , const QString & )>(_a, &GroupChatController::groupChatJoiningFailed, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & , const QString & )>(_a, &GroupChatController::contactInvitedToGroupChat, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)()>(_a, &GroupChatController::groupChatInviteeSelectionNeeded, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & )>(_a, &GroupChatController::groupChatLeft, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & , const QString & )>(_a, &GroupChatController::groupChatLeavingFailed, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & )>(_a, &GroupChatController::groupChatDeleted, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & , const QString & )>(_a, &GroupChatController::groupChatDeletionFailed, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & )>(_a, &GroupChatController::groupChatMadePrivate, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const QString & )>(_a, &GroupChatController::groupChatMadePublic, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const GroupChatUser & )>(_a, &GroupChatController::userDisallowedOrUnbanned, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const GroupChatUser & )>(_a, &GroupChatController::userAllowedOrBanned, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const GroupChatUser & )>(_a, &GroupChatController::participantReceived, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (GroupChatController::*)(const GroupChatUser & )>(_a, &GroupChatController::participantLeft, 19))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->busy(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->groupChatParticipationSupported(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->groupChatCreationSupported(); break;
        default: break;
        }
    }
}

const QMetaObject *GroupChatController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GroupChatController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19GroupChatControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GroupChatController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 29;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void GroupChatController::busyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void GroupChatController::groupChatParticipationSupportedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GroupChatController::groupChatServicesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GroupChatController::privateGroupChatCreationFailed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void GroupChatController::publicGroupChatCreationFailed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void GroupChatController::groupChatCreated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void GroupChatController::groupChatJoined(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void GroupChatController::groupChatJoiningFailed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void GroupChatController::contactInvitedToGroupChat(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void GroupChatController::groupChatInviteeSelectionNeeded()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void GroupChatController::groupChatLeft(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void GroupChatController::groupChatLeavingFailed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void GroupChatController::groupChatDeleted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void GroupChatController::groupChatDeletionFailed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void GroupChatController::groupChatMadePrivate(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void GroupChatController::groupChatMadePublic(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void GroupChatController::userDisallowedOrUnbanned(const GroupChatUser & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void GroupChatController::userAllowedOrBanned(const GroupChatUser & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void GroupChatController::participantReceived(const GroupChatUser & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void GroupChatController::participantLeft(const GroupChatUser & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}
QT_WARNING_POP
