/****************************************************************************
** Meta object code from reading C++ file 'Message.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kaidan-0.16.0/src/Message.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Message.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4FileE_t {};
} // unnamed namespace

template <> constexpr inline auto File::qt_create_metaobjectdata<qt_meta_tag_ZN4FileE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "File",
        "fileId",
        "fileGroupId",
        "name",
        "description",
        "mimeTypeName",
        "mimeTypeIcon",
        "formattedSize",
        "lastModified",
        "displayInline",
        "downloadUrl",
        "QUrl",
        "locallyAvailable",
        "localFilePath",
        "localFileUrl",
        "externalId",
        "hasThumbnail",
        "type",
        "Enums::MessageType",
        "details",
        "isNew",
        "transferOutgoing",
        "transferState",
        "TransferState",
        "Pending",
        "Transferring",
        "CanceledByUser",
        "Failed",
        "Done"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fileId'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'fileGroupId'
        QtMocHelpers::PropertyData<qint64>(2, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'mimeTypeName'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'mimeTypeIcon'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'formattedSize'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'lastModified'
        QtMocHelpers::PropertyData<QDateTime>(8, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'displayInline'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'downloadUrl'
        QtMocHelpers::PropertyData<QUrl>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'locallyAvailable'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'localFilePath'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'localFileUrl'
        QtMocHelpers::PropertyData<QUrl>(14, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'externalId'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'hasThumbnail'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'type'
        QtMocHelpers::PropertyData<Enums::MessageType>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'details'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isNew'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'transferOutgoing'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'transferState'
        QtMocHelpers::PropertyData<enum TransferState>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TransferState'
        QtMocHelpers::EnumData<enum TransferState>(23, 23, QMC::EnumIsScoped).add({
            {   24, TransferState::Pending },
            {   25, TransferState::Transferring },
            {   26, TransferState::CanceledByUser },
            {   27, TransferState::Failed },
            {   28, TransferState::Done },
        }),
    };
    return QtMocHelpers::metaObjectData<File, qt_meta_tag_ZN4FileE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN4FileE[] = {
    QMetaObject::SuperData::link<Enums::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject File::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4FileE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4FileE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN4FileE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4FileE_t>.metaTypes,
    nullptr
} };

void File::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<File *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->fileId(); break;
        case 1: *reinterpret_cast<qint64*>(_v) = _t->fileGroupId; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->_name(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->_description(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->mimeTypeName(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->mimeTypeIcon(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->formattedSize(); break;
        case 7: *reinterpret_cast<QDateTime*>(_v) = _t->lastModified; break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->displayInline(); break;
        case 9: *reinterpret_cast<QUrl*>(_v) = _t->downloadUrl(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->locallyAvailable(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->localFilePath; break;
        case 12: *reinterpret_cast<QUrl*>(_v) = _t->localFileUrl(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->externalId; break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->hasThumbnail(); break;
        case 15: *reinterpret_cast<Enums::MessageType*>(_v) = _t->type(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->details(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->isNew; break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->transferOutgoing; break;
        case 19: *reinterpret_cast<enum TransferState*>(_v) = _t->transferState; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: QtMocHelpers::setProperty(_t->fileGroupId, *reinterpret_cast<qint64*>(_v)); break;
        case 7: QtMocHelpers::setProperty(_t->lastModified, *reinterpret_cast<QDateTime*>(_v)); break;
        case 11: QtMocHelpers::setProperty(_t->localFilePath, *reinterpret_cast<QString*>(_v)); break;
        case 13: QtMocHelpers::setProperty(_t->externalId, *reinterpret_cast<QString*>(_v)); break;
        case 17: QtMocHelpers::setProperty(_t->isNew, *reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN28MessageReactionDeliveryStateE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageReactionDeliveryState::qt_create_metaobjectdata<qt_meta_tag_ZN28MessageReactionDeliveryStateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageReactionDeliveryState",
        "Enum",
        "PendingAddition",
        "PendingRemovalAfterSent",
        "PendingRemovalAfterDelivered",
        "ErrorOnAddition",
        "ErrorOnRemovalAfterSent",
        "ErrorOnRemovalAfterDelivered",
        "Sent",
        "Delivered"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Enum'
        QtMocHelpers::EnumData<enum Enum>(1, 1, QMC::EnumFlags{}).add({
            {    2, Enum::PendingAddition },
            {    3, Enum::PendingRemovalAfterSent },
            {    4, Enum::PendingRemovalAfterDelivered },
            {    5, Enum::ErrorOnAddition },
            {    6, Enum::ErrorOnRemovalAfterSent },
            {    7, Enum::ErrorOnRemovalAfterDelivered },
            {    8, Enum::Sent },
            {    9, Enum::Delivered },
        }),
    };
    return QtMocHelpers::metaObjectData<MessageReactionDeliveryState, qt_meta_tag_ZN28MessageReactionDeliveryStateE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageReactionDeliveryState::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28MessageReactionDeliveryStateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28MessageReactionDeliveryStateE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28MessageReactionDeliveryStateE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN15MessageReactionE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageReaction::qt_create_metaobjectdata<qt_meta_tag_ZN15MessageReactionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageReaction",
        "emoji",
        "deliveryState",
        "MessageReactionDeliveryState::Enum"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'emoji'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'deliveryState'
        QtMocHelpers::PropertyData<MessageReactionDeliveryState::Enum>(2, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageReaction, qt_meta_tag_ZN15MessageReactionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN15MessageReactionE[] = {
    QMetaObject::SuperData::link<MessageReactionDeliveryState::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject MessageReaction::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageReactionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MessageReactionE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN15MessageReactionE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MessageReactionE_t>.metaTypes,
    nullptr
} };

void MessageReaction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<MessageReaction *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->emoji; break;
        case 1: *reinterpret_cast<MessageReactionDeliveryState::Enum*>(_v) = _t->deliveryState; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->emoji, *reinterpret_cast<QString*>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->deliveryState, *reinterpret_cast<MessageReactionDeliveryState::Enum*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN7MessageE_t {};
} // unnamed namespace

template <> constexpr inline auto Message::qt_create_metaobjectdata<qt_meta_tag_ZN7MessageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Message",
        "relevantId",
        "formattedTimestamp",
        "encryption",
        "Encryption::Enum",
        "deliveryState",
        "DeliveryState",
        "body",
        "TrustLevel",
        "Untrusted",
        "Trusted",
        "Authenticated"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'relevantId'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'formattedTimestamp'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'encryption'
        QtMocHelpers::PropertyData<Encryption::Enum>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'deliveryState'
        QtMocHelpers::PropertyData<DeliveryState>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'body'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TrustLevel'
        QtMocHelpers::EnumData<enum TrustLevel>(8, 8, QMC::EnumIsScoped).add({
            {    9, TrustLevel::Untrusted },
            {   10, TrustLevel::Trusted },
            {   11, TrustLevel::Authenticated },
        }),
    };
    return QtMocHelpers::metaObjectData<Message, qt_meta_tag_ZN7MessageE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN7MessageE[] = {
    QMetaObject::SuperData::link<Encryption::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject Message::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7MessageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7MessageE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN7MessageE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7MessageE_t>.metaTypes,
    nullptr
} };

void Message::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<Message *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->relevantId(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->formattedTimestamp(); break;
        case 2: *reinterpret_cast<Encryption::Enum*>(_v) = _t->encryption; break;
        case 3: *reinterpret_cast<DeliveryState*>(_v) = _t->deliveryState; break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->m_body; break;
        default: break;
        }
    }
}
QT_WARNING_POP
